@@ -5,7 +5,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: xtr_textsubs.cpp 3024 2005-08-17 11:46:09Z mosu $
+   $Id: xtr_textsubs.cpp 3377 2006-11-23 15:56:44Z mosu $
 
    extracts tracks from Matroska files into other files
 
@@ -40,35 +40,39 @@ xtr_srt_c::create_file(xtr_base_c *_master,
   out->write_bom(sub_charset);
 }
 
+#define LLD02 "%02" PRId64
+#define LLD03 "%03" PRId64
+
 void
-xtr_srt_c::handle_block(KaxBlock &block,
+xtr_srt_c::handle_frame(memory_cptr &frame,
                         KaxBlockAdditions *additions,
                         int64_t timecode,
                         int64_t duration,
                         int64_t bref,
-                        int64_t fref) {
+                        int64_t fref,
+                        bool keyframe,
+                        bool discardable,
+                        bool references_valid) {
   char *text;
   int64_t start, end;
   string buffer;
 
   if (-1 == duration) {
-    mxwarn("Track %lld: Subtitle entry number %d is missing its duration. "
+    mxwarn("Track " LLD ": Subtitle entry number %d is missing its duration. "
            "Assuming a duration of 1s.\n", tid, num_entries + 1);
     duration = 1000000000;
   }
 
-  DataBuffer &data = block.GetBuffer(0);
-
   start = timecode / 1000000;
   end = start + duration / 1000000;
 
   ++num_entries;
-  text = new char[data.Size() + 1];
-  memcpy(text, data.Buffer(), data.Size());
-  text[data.Size()] = 0;
+  text = new char[frame->get_size() + 1];
+  memcpy(text, frame->get(), frame->get_size());
+  text[frame->get_size()] = 0;
   buffer = mxsprintf("%d\n"
-                     "%02lld:%02lld:%02lld,%03lld --> "
-                     "%02lld:%02lld:%02lld,%03lld\n"
+                     LLD02 ":" LLD02 ":" LLD02 "," LLD03 " --> "
+                     LLD02 ":" LLD02 ":" LLD02 "," LLD03 "\n"
                      "%s\n\n",
                      num_entries,
                      start / 1000 / 60 / 60, (start / 1000 / 60) % 60,
@@ -99,7 +103,8 @@ xtr_ssa_c::xtr_ssa_c(const string &_codec_id,
 void
 xtr_ssa_c::create_file(xtr_base_c *_master,
                        KaxTrackEntry &track) {
-  char *s, *p1;
+  char *s;
+  const char *p1;
   const unsigned char *pd;
   int bom_len, pos1, pos2, priv_size;
   string sconv;
@@ -107,8 +112,9 @@ xtr_ssa_c::create_file(xtr_base_c *_master,
 
   priv = FINDFIRST(&track, KaxCodecPrivate);
   if (NULL == priv)
-    mxerror("Track %lld with the CodecID '%s' is missing the \"codec private"
-            "\" element and cannot be extracted.\n", tid, codec_id.c_str());
+    mxerror("Track " LLD " with the CodecID '%s' is missing the \"codec "
+            "private \" element and cannot be extracted.\n", tid,
+            codec_id.c_str());
 
   xtr_base_c::create_file(_master, track);
   out->write_bom(sub_charset);
@@ -173,12 +179,15 @@ xtr_ssa_c::create_file(xtr_base_c *_master,
 }
 
 void
-xtr_ssa_c::handle_block(KaxBlock &block,
+xtr_ssa_c::handle_frame(memory_cptr &frame,
                         KaxBlockAdditions *additions,
                         int64_t timecode,
                         int64_t duration,
                         int64_t bref,
-                        int64_t fref) {
+                        int64_t fref,
+                        bool keyframe,
+                        bool discardable,
+                        bool references_valid) {
   int i, k, num;
   char *s;
   vector<string> fields;
@@ -187,7 +196,7 @@ xtr_ssa_c::handle_block(KaxBlock &block,
   int64_t start, end;
 
   if (0 > duration) {
-    mxwarn("Subtitle track %lld is missing some duration elements. "
+    mxwarn("Subtitle track " LLD " is missing some duration elements. "
            "Please check the resulting SSA/ASS file for entries that "
            "have the same start and end time.\n", tid);
     warning_printed = true;
@@ -195,10 +204,10 @@ xtr_ssa_c::handle_block(KaxBlock &block,
   start = timecode / 1000000;
   end = start + duration / 1000000;
 
-  DataBuffer &data = block.GetBuffer(0);
-  autofree_ptr<char> af_s(s = (char *)safemalloc(data.Size() + 1));
-  memcpy(s, data.Buffer(), data.Size());
-  s[data.Size()] = 0;
+  s = (char *)safemalloc(frame->get_size() + 1);
+  memory_c af_s((unsigned char *)s, 0, true);
+  memcpy(s, frame->get(), frame->get_size());
+  s[frame->get_size()] = 0;
 
   // Split the line into the fields.
   // Specs say that the following fields are to put into the block:
@@ -235,18 +244,20 @@ xtr_ssa_c::handle_block(KaxBlock &block,
 
   line = "Dialogue: ";
   for (i = 0; i < ssa_format.size(); i++) {
-    string &format = ssa_format[i];
+    string format = ssa_format[i];
 
+    if (downcase(format) == "actor")
+      format = "name";
     if (0 < i)
       line += ",";
     if (format == "marked")
       line += "Marked=0";
     else if (format == "start")
-      line += mxsprintf("%lld:%02lld:%02lld.%02lld",
+      line += mxsprintf(LLD ":" LLD02 ":" LLD02 "." LLD02,
                         start / 1000 / 60 / 60, (start / 1000 / 60) % 60,
                         (start / 1000) % 60, (start % 1000) / 10);
     else if (format == "end")
-      line += mxsprintf("%lld:%02lld:%02lld.%02lld",
+      line += mxsprintf(LLD ":" LLD02 ":" LLD02 "." LLD02,
                         end / 1000 / 60 / 60, (end / 1000 / 60) % 60,
                         (end / 1000) % 60, (end % 1000) / 10);
     else {
@@ -296,28 +307,21 @@ xtr_usf_c::create_file(xtr_base_c *_master,
                        KaxTrackEntry &track) {
   KaxCodecPrivate *priv;
   KaxTrackLanguage *language;
-  unsigned char *new_priv;
-  uint32_t private_size;
 
   priv = FINDFIRST(&track, KaxCodecPrivate);
   if (NULL == priv)
-    mxerror("Track %lld with the CodecID '%s' is missing the \"codec private"
-            "\" element and cannot be extracted.\n", tid, codec_id.c_str());
+    mxerror("Track " LLD " with the CodecID '%s' is missing the \"codec "
+            "private \" element and cannot be extracted.\n", tid,
+            codec_id.c_str());
 
   if (!content_decoder.initialize(track))
     mxerror("Tracks with unsupported content encoding schemes (compression "
             "or encryption) cannot be extracted.\n");
 
-  private_size = priv->GetSize();
-
-  new_priv = const_cast<unsigned char *>(priv->GetBuffer());
-  if (!content_decoder.reverse(new_priv, private_size,
-                               CONTENT_ENCODING_SCOPE_CODECPRIVATE)) {
-    m_codec_private.append((const char *)new_priv, private_size);
-  } else {
-    m_codec_private.append((const char *)new_priv, private_size);
-    safefree(new_priv);
-  }
+  memory_cptr new_priv(new memory_c(priv->GetBuffer(), priv->GetSize(),
+                                    false));
+  content_decoder.reverse(new_priv, CONTENT_ENCODING_SCOPE_CODECPRIVATE);
+  m_codec_private.append((const char *)new_priv->get(), new_priv->get_size());
 
   language = FINDFIRST(&track, KaxTrackLanguage);
   if (NULL == language)
@@ -330,15 +334,15 @@ xtr_usf_c::create_file(xtr_base_c *_master,
 
     usf_master = dynamic_cast<xtr_usf_c *>(_master);
     if (NULL == usf_master)
-      mxerror("Cannot write track %lld with the CodecID '%s' to the file '%s' "
-              "because track %lld with the CodecID '%s' is already being "
-              "written to the same file.\n", tid, codec_id.c_str(),
+      mxerror("Cannot write track " LLD " with the CodecID '%s' to the file "
+              "'%s' because track " LLD " with the CodecID '%s' is already "
+              "being written to the same file.\n", tid, codec_id.c_str(),
               file_name.c_str(), _master->tid, _master->codec_id.c_str());
     if (m_codec_private != usf_master->m_codec_private)
-      mxerror("Cannot write track %lld with the CodecID '%s' to the file '%s' "
-              "because track %lld with the CodecID '%s' is already being "
-              "written to the same file, and their CodecPrivate data (the "
-              "USF styles etc) do not match.\n", tid, codec_id.c_str(),
+      mxerror("Cannot write track " LLD " with the CodecID '%s' to the file "
+              "'%s' because track " LLD " with the CodecID '%s' is already "
+              "being written to the same file, and their CodecPrivate data "
+              "(the USF styles etc) do not match.\n", tid, codec_id.c_str(),
               file_name.c_str(), _master->tid, _master->codec_id.c_str());
 
     m_formatter = usf_master->m_formatter;
@@ -366,32 +370,26 @@ xtr_usf_c::create_file(xtr_base_c *_master,
       mxerror("Failed to create the file '%s': %d (%s)\n", file_name.c_str(),
               errno, strerror(errno));
     } catch (xml_formatter_error_c &error) {
-      mxerror("Failed to parse the USF codec private data for track %lld: "
+      mxerror("Failed to parse the USF codec private data for track " LLD ": "
               "%s\n", tid, error.get_error().c_str());
-      
+
     }
   }
 }
 
 void
-xtr_usf_c::handle_block(KaxBlock &block,
+xtr_usf_c::handle_frame(memory_cptr &frame,
                         KaxBlockAdditions *additions,
                         int64_t timecode,
                         int64_t duration,
                         int64_t bref,
-                        int64_t fref) {
-  if (0 == block.NumberFrames())
-    return;
-
-  DataBuffer &data_buffer = block.GetBuffer(0);
+                        int64_t fref,
+                        bool keyframe,
+                        bool discardable,
+                        bool references_valid) {
   usf_entry_t entry("", timecode, timecode + duration);
-  uint32_t size = data_buffer.Size();
-  unsigned char *data = data_buffer.Buffer();
-  bool free_mem = content_decoder.reverse(data, size,
-                                          CONTENT_ENCODING_SCOPE_BLOCK);
-  entry.m_text.append((const char *)data, size);
-  if (free_mem)
-    safefree(data);
+  content_decoder.reverse(frame, CONTENT_ENCODING_SCOPE_BLOCK);
+  entry.m_text.append((const char *)frame->get(), frame->get_size());
   m_entries.push_back(entry);
 }
 
@@ -402,7 +400,7 @@ xtr_usf_c::finish_track() {
   try {
     m_formatter->format(mxsprintf("<subtitles>\n<language code=\"%s\"/>\n",
                                   m_language.c_str()));
-    foreach(entry, m_entries) {
+    mxforeach(entry, m_entries) {
       string text = entry->m_text;
       strip(text, true);
       m_formatter->format(mxsprintf("<subtitle start=\"" FMT_TIMECODE
@@ -414,8 +412,8 @@ xtr_usf_c::finish_track() {
     }
     m_formatter->format("</subtitles>\n");
   } catch (xml_formatter_error_c &error) {
-    mxerror("Failed to parse an USF subtitle entry for track %lld: %s\n", tid,
-            error.get_error().c_str());
+    mxerror("Failed to parse an USF subtitle entry for track " LLD ": %s\n",
+            tid, error.get_error().c_str());
   }
 }
 
@@ -427,7 +425,7 @@ xtr_usf_c::finish_file() {
       out->printf("\n");
     }
   } catch (xml_formatter_error_c &error) {
-    mxerror("Failed to parse the USF end tag for track %lld: %s\n", tid,
+    mxerror("Failed to parse the USF end tag for track " LLD ": %s\n", tid,
             error.get_error().c_str());
   }
 }
