@@ -6,7 +6,7 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: p_video.h 2988 2005-07-19 17:31:43Z mosu $
+   $Id: p_video.h 3469 2007-01-15 19:02:18Z mosu $
 
    class definition for the video output module
 
@@ -57,6 +57,7 @@ protected:
 class mpeg1_2_video_packetizer_c: public video_packetizer_c {
 protected:
   M2VParser parser;
+  memory_cptr seq_hdr;
   bool framed, aspect_ratio_extracted;
 
 public:
@@ -72,6 +73,7 @@ protected:
   virtual void extract_fps(const unsigned char *buffer, int size);
   virtual void extract_aspect_ratio(const unsigned char *buffer, int size);
   virtual void create_private_data();
+  virtual int process_framed(packet_cptr packet);
 };
 
 class mpeg4_p2_video_packetizer_c: public video_packetizer_c {
@@ -80,6 +82,7 @@ protected:
   deque<int64_t> available_timecodes, available_durations;
   int64_t timecodes_generated, last_i_p_frame, previous_timecode;
   bool aspect_ratio_extracted, input_is_native, output_is_native;
+  bool size_extracted;
 
 public:
   mpeg4_p2_video_packetizer_c(generic_reader_c *_reader,
@@ -95,6 +98,7 @@ protected:
   virtual void flush_frames_maybe(frame_type_e next_frame);
   virtual void flush_frames(bool end_of_file);
   virtual void extract_aspect_ratio(const unsigned char *buffer, int size);
+  virtual void extract_size(const unsigned char *buffer, int size);
   virtual void fix_codec_string();
   virtual void handle_missing_timecodes(bool end_of_file);
 };
@@ -106,8 +110,25 @@ public:
                                track_info_c &_ti);
   virtual int process(packet_cptr packet);
 
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src,
+                                             string &error_message);
+
 protected:
   virtual void extract_aspect_ratio();
 };
 
+// class mpeg4_p10_es_video_packetizer_c: public video_packetizer_c {
+// public:
+//   mpeg4_p10_es_video_packetizer_c(generic_reader_c *_reader,
+//                                   double _fps, int _width, int _height,
+//                                   track_info_c &_ti);
+//   virtual int process(packet_cptr packet);
+
+//   virtual connection_result_e can_connect_to(generic_packetizer_c *src,
+//                                              string &error_message);
+
+// protected:
+//   virtual void extract_aspect_ratio();
+// };
+
 #endif // __P_VIDEO_H
